# -*- mode: makefile; coding: utf-8 -*-
# Copyright © 2003 Colin Walters <walters@debian.org>
# Copyright © 2006 Marc Dequènes (Duck) <Duck@DuckCorp.org>
# Copyright © 2006,2007 Jonas Smedegaard <dr@jones.dk>
# Description: manage Python modules using the 'distutils' build system
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02111-1307 USA.
#

# This class provides rules for old and new Python policy.  Leaving
# DEB_PYTHON_SYSTEM unset selects the old behavior.  The pysupport and
# pycentral methods are available to comply with new policy.  Don't forget
# to update your 'debian/control'.  (Build-Depends are correctly generated
# if you are using the auto control generation feature.)


_cdbs_scripts_path ?= /usr/lib/cdbs
_cdbs_rules_path ?= /usr/share/cdbs/1/rules
_cdbs_class_path ?= /usr/share/cdbs/1/class

ifndef _cdbs_class_python_distutils
_cdbs_class_python_distutils = 1

include $(_cdbs_rules_path)/buildcore.mk$(_cdbs_makefile_suffix)
include $(_cdbs_class_path)/langcore.mk$(_cdbs_makefile_suffix)


# common variables
DEB_PYTHON_SETUP_CMD = setup.py
DEB_PYTHON_CLEAN_ARGS = -a
DEB_PYTHON_BUILD_ARGS = --build-base="$(CURDIR)/$(DEB_BUILDDIR)/build"
DEB_PYTHON_INSTALL_ARGS_ALL = --no-compile -O0

# We always install into target package, unlike DEB_DESTDIR
ifndef DEB_PYTHON_SYSTEM
DEB_PYTHON_DESTDIR = $(CURDIR)/debian/$(cdbs_curpkg)
else
# Arrrrgh - except for new python policy that broke the promise! :-(
DEB_PYTHON_DESTDIR = $(DEB_DESTDIR)
endif


ifndef DEB_PYTHON_SYSTEM
########################### old policy method ###########################

$(warning ########################################################)
$(warning Your package does not conform to the new Python policy.)
$(warning Please consider updating.  Here is some documentation:)
$(warning   http://wiki.debian.org/DebianPython/NewPolicy)
$(warning   http://wiki.debian.org/DebianPythonFAQ)
$(warning ########################################################)

DEB_PYTHON_COMPILE_VERSION = 
DEB_PYTHON_VERSIONS = 2.1 2.2 2.3 2.4 2.5

cdbs_python_current_version := $(shell /usr/bin/python -V 2>&1 | sed 's/^[^0-9]*\([0-9]\+.[0-9]\+\)[^0-9].*/\1/')
cdbs_python_next_version := $(shell expr '$(cdbs_python_current_version)' : '[^0-9]*\([0-9]\+\).').$(shell expr '(' '$(cdbs_python_current_version)' : '[^0-9]*[0-9]\+.\([0-9]\+\)' ')' + 1)

# This variable is deprecated.
DEB_PYTHON_REAL_LIB_PACKAGES = $(strip $(filter $(patsubst %,python%%,$(DEB_PYTHON_VERSIONS)),$(filter-out $(DEB_PYTHON_PACKAGES_EXCLUDE),$(DEB_ALL_PACKAGES))))
_cdbs_deprecated_vars += DEB_PYTHON_REAL_LIB_PACKAGES
_cdbs_deprecated_DEB_PYTHON_REAL_LIB_PACKAGES_default := $(DEB_PYTHON_REAL_LIB_PACKAGES)
# New in 0.x.y.
DEB_PYTHON_MODULE_PACKAGES = $(DEB_PYTHON_REAL_LIB_PACKAGES)

DEB_PYTHON_MODULE_ARCH_PACKAGES = $(filter $(DEB_PYTHON_MODULE_PACKAGES), $(DEB_ARCH_PACKAGES))
DEB_PYTHON_MODULE_INDEP_PACKAGES = $(filter $(DEB_PYTHON_MODULE_PACKAGES), $(DEB_INDEP_PACKAGES))

# If no versioned python library packages found, grab all simpler ones
ifeq (,$(DEB_PYTHON_MODULE_ARCH_PACKAGES))
# This variable is deprecated.
DEB_PYTHON_SIMPLE_PACKAGES = $(strip $(filter python-%,$(filter-out $(DEB_PYTHON_PACKAGES_EXCLUDE),$(DEB_ALL_PACKAGES))))
_cdbs_deprecated_vars += DEB_PYTHON_SIMPLE_PACKAGES
_cdbs_deprecated_DEB_PYTHON_SIMPLE_PACKAGES_default := $(DEB_PYTHON_SIMPLE_PACKAGES)
# New in 0.x.y.
DEB_PYTHON_MODULE_PACKAGES = $(DEB_PYTHON_SIMPLE_PACKAGES)
endif

ifeq (,$(DEB_PYTHON_MODULE_ARCH_PACKAGES))
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), python-dev (>= $(cdbs_python_current_version)), python-dev (<< $(cdbs_python_next_version))
else
cdbs_python_build_versions := $(filter-out -%,$(subst -, -,$(patsubst python%,%,$(DEB_PYTHON_MODULE_ARCH_PACKAGES))))
comma := ,
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), $(patsubst %,python%-dev$(comma),$(cdbs_python_build_versions)) python
endif

cdbs_python_compile_version = $(DEB_PYTHON_COMPILE_VERSION)
cdbs_python_curpkg_build_versions = $(filter-out -%,$(subst -, -,$(patsubst python%,%,$(cdbs_curpkg))))

# This class can optionally utilize debhelper's "dh_python" command.  Just
# be sure you include debhelper.mk before including this file.
ifdef _cdbs_rules_debhelper
ifneq (,$(strip $(filter 2.4,$(cdbs_python_build_versions))))
# Python 2.4 support
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), debhelper (>= 4.2.28)
else
# Python 2.3 support
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), debhelper (>= 4.1.60)
endif

DEB_DH_PYTHON_ARGS = $(addprefix -V ,$(cdbs_python_compile_version)) $(DEB_DH_PYTHON_ARGS_ALL) $(DEB_DH_PYTHON_ARGS_$(cdbs_curpkg))
endif


else
########################## new policy methods ###########################

# This variable is deprecated.
DEB_PYTHON_MODULE_PACKAGE = $(firstword $(filter-out %-doc %-dev %-common, $(DEB_PACKAGES)))
_cdbs_deprecated_vars += DEB_PYTHON_MODULE_PACKAGE
_cdbs_deprecated_DEB_PYTHON_MODULE_PACKAGE_default := $(DEB_PYTHON_MODULE_PACKAGE)
# New in 0.x.y.
# TODO: Use DEB_PYTHON_MODU*LE_PACKAGE but without firstword function (requires abi bump!)
DEB_PYTHON_MODULE_PACKAGES = $(DEB_PYTHON_MODULE_PACKAGE)

DEB_PYTHON_MODULE_ARCH_PACKAGES = $(filter $(DEB_PYTHON_MODULE_PACKAGES), $(DEB_ARCH_PACKAGES))
DEB_PYTHON_MODULE_INDEP_PACKAGES = $(filter $(DEB_PYTHON_MODULE_PACKAGES), $(DEB_INDEP_PACKAGES))

# This variable is deprecated
## FIXME: Die if used together with pycentral, that does not support it!
## FIXME: Warn if used without debhelper: is applied with dh_pysupport.
## TODO: Check if usable also with old python policy: was added to deprecated dh_python too.
DEB_PYTHON_PRIVATE_MODULES_DIRS =
_cdbs_deprecated_vars += DEB_PYTHON_PRIVATE_MODULES_DIRS

# check python system
cdbs_use_xs_field := $(shell grep -q "^XS-Python-Version:" debian/control && echo yes)
cdbs_selected_pycompat := $(shell if [ -e debian/pycompat ]; then cat debian/pycompat; fi)
cdbs_pycompat = $(cdbs_selected_pycompat)
ifeq (pysupport, $(DEB_PYTHON_SYSTEM))
## FIXME: Arrrrgh - new policy works only with a single python module per source package!
  cdbs_python_support_path = usr/share/python-support/$(DEB_PYTHON_MODULE_PACKAGES)
  ifeq (, $(cdbs_selected_pycompat))
    cdbs_pycompat = 2
  endif # use pycompat
  # warning pysupport compatibility mode
  ifneq (, $(cdbs_use_xs_field))
    $(warning Use of XS-Python-Version and XB-Python-Version fields in 'debian/control' is deprecated with pysupport method, use 'debian/pyversions' if you need to specify specific versions)
  endif # use XS field (compat)
else
  ifeq (pycentral, $(DEB_PYTHON_SYSTEM))
    ifeq (, $(cdbs_selected_pycompat))
      cdbs_pycompat = 2
    endif # use pycompat
  else
    ifneq (, $(DEB_PYTHON_SYSTEM))
      $(error unsupported Python system: $(DEB_PYTHON_SYSTEM) (select either pysupport or pycentral))
    else
      ifneq (, $(cdbs_use_xs_field))
        $(error Your package uses the new Python policy; you must set DEB_PYTHON_SYSTEM to "pysupport" or "pycentral".)
      endif
      ifneq (, $(cdbs_selected_pycompat))
        ifeq (yes, $(shell expr $(cdbs_selected_pycompat) \> 1 >/dev/null && echo yes))
          $(error Your package uses the new Python policy; you must set DEB_PYTHON_SYSTEM to "pysupport" or "pycentral".)
        endif
      endif # use pycompat
    endif # unknown method
  endif # pycentral
endif # pysupport

# Calculate cdbs_python_build_versions
## FIXME: Arrrrgh - new policy works only with a single python module per source package!
cdbs_python_current_version := $(shell pyversions -vd)
ifeq (,$(DEB_PYTHON_MODULE_ARCH_PACKAGES))
  # check if current is in build versions
  ifneq ($(cdbs_python_current_version), $(filter $(cdbs_python_current_version), $(shell pyversions -vr)))
    cdbs_python_compile_version := $(firstword $(strip $(sort $(shell pyversions -vr))))
    cdbs_python_build_versions := $(cdbs_python_compile_version)
  else
    cdbs_python_build_versions := $(cdbs_python_current_version)
  endif
else
cdbs_python_build_versions := $(shell pyversions -vr)
endif # archall

# check if build is possible
ifeq (, $(cdbs_python_build_versions))
ifeq (pysupport, $(DEB_PYTHON_SYSTEM))
$(error invalid setting in 'debian/pyversions')
else
$(error invalid setting for XS-Python-Version)
endif # system selected
endif # build versions empty


# Declare Build-Deps for packages using this file
# This class can optionally utilize debhelper's "dh_python" command.  Just
# be sure you include debhelper.mk before including this file.
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), cdbs (>= 0.4.43)
ifdef _cdbs_rules_debhelper
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), debhelper (>= 5.0.37.2)
endif
ifeq (,$(DEB_PYTHON_MODULE_ARCH_PACKAGES))
  ifneq (, $(cdbs_python_compile_version))
    CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), python$(cdbs_python_compile_version)-dev, python (>= 2.3.5-11)
  else
    CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), python-dev (>= 2.3.5-11)
  endif
else
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), python-all-dev (>= 2.3.5-11)
endif
ifeq (pysupport, $(DEB_PYTHON_SYSTEM))
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), python-support (>= 0.3.2)
else
CDBS_BUILD_DEPENDS := $(CDBS_BUILD_DEPENDS), python-central (>= 0.5.0)
endif

# TODO: Support multiple python modules built for different python versions
cdbs_python_curpkg_build_versions = $(cdbs_python_build_versions)

DEB_DH_PYTHON_ARGS = $(DEB_DH_PYTHON_ARGS_ALL) $(DEB_DH_PYTHON_ARGS_$(cdbs_curpkg)) $(DEB_PYTHON_PRIVATE_MODULES_DIRS) $(DEB_PYTHON_PRIVATE_MODULES_DIRS_$(cdbs_curpkg))

endif


########################## all policy methods ###########################

pre-build::
	mkdir -p python-module-stamp


# build stage
$(patsubst %,build/%,$(DEB_PYTHON_MODULE_INDEP_PACKAGES) $(DEB_PYTHON_MODULE_ARCH_PACKAGES)) :: build/% : python-module-stamp/%

$(patsubst %,python-module-stamp/%,$(DEB_PYTHON_MODULE_INDEP_PACKAGES)) :: python-module-stamp/%:
	cd $(DEB_SRCDIR) && /usr/bin/python$(cdbs_python_compile_version) $(DEB_PYTHON_SETUP_CMD) build $(DEB_PYTHON_BUILD_ARGS)
	touch $@

$(patsubst %,python-module-stamp/%,$(DEB_PYTHON_MODULE_ARCH_PACKAGES)) :: python-module-stamp/%:
	for buildver in $(cdbs_python_curpkg_build_versions); do \
		cd $(DEB_SRCDIR) && /usr/bin/python$$buildver $(DEB_PYTHON_SETUP_CMD) build $(DEB_PYTHON_BUILD_ARGS); \
	done
	touch $@


# install stage
$(patsubst %,install/%,$(DEB_PYTHON_MODULE_INDEP_PACKAGES)) :: install/%:
	cd $(DEB_SRCDIR) && /usr/bin/python$(cdbs_python_compile_version) $(DEB_PYTHON_SETUP_CMD) install --root="$(DEB_PYTHON_DESTDIR)" $(DEB_PYTHON_INSTALL_ARGS_ALL) $(DEB_PYTHON_INSTALL_ARGS_$(cdbs_curpkg))

$(patsubst %,install/%,$(DEB_PYTHON_MODULE_ARCH_PACKAGES)) :: install/%:
	for buildver in $(cdbs_python_curpkg_build_versions); do \
		cd $(DEB_SRCDIR) && /usr/bin/python$$buildver $(DEB_PYTHON_SETUP_CMD) install --root="$(DEB_PYTHON_DESTDIR)" $(DEB_PYTHON_INSTALL_ARGS_ALL) $(DEB_PYTHON_INSTALL_ARGS_$(cdbs_curpkg)); \
	done


# This class can optionally use debhelper's commands.  Just
# be sure you include debhelper.mk before including this file.
ifdef _cdbs_rules_debhelper

$(patsubst %,binary-install/%,$(DEB_PACKAGES)) :: binary-install/%:
ifdef DEB_PYTHON_SYSTEM
ifeq (pysupport, $(DEB_PYTHON_SYSTEM))
	dh_pysupport -p$(cdbs_curpkg) $(DEB_PYTHON_PRIVATE_MODULES_DIRS) $(DEB_PYTHON_PRIVATE_MODULES_DIRS_$(cdbs_curpkg))
else
	dh_pycentral -p$(cdbs_curpkg)
endif
endif
	dh_python -p$(cdbs_curpkg) $(DEB_DH_PYTHON_ARGS)
endif


# clean stage
clean:: $(patsubst %,python-module-clean/%,$(DEB_PYTHON_MODULE_INDEP_PACKAGES) $(DEB_PYTHON_MODULE_ARCH_PACKAGES))

$(patsubst %,python-module-clean/%,$(DEB_PYTHON_MODULE_INDEP_PACKAGES)) :: python-module-clean/%:
	-cd $(DEB_SRCDIR) && /usr/bin/python$(cdbs_python_compile_version) $(DEB_PYTHON_SETUP_CMD) clean $(DEB_PYTHON_CLEAN_ARGS)

$(patsubst %,python-module-clean/%,$(DEB_PYTHON_MODULE_ARCH_PACKAGES)) :: python-module-clean/%:
	-for buildver in $(cdbs_python_curpkg_build_versions); do \
		cd $(DEB_SRCDIR) && /usr/bin/python$$buildver $(DEB_PYTHON_SETUP_CMD) clean $(DEB_PYTHON_CLEAN_ARGS); \
	done

clean::
ifdef DEB_PYTHON_SYSTEM
ifeq (, $(cdbs_selected_pycompat))
	echo "$(cdbs_pycompat)" >debian/pycompat
endif # use pycompat
endif
	rm -rf python-build-stamp

# Calling setup.py clean may create .pyc files, so we need a final cleanup
# pass here.
clean::
	find . -name '*.pyc' -exec rm '{}' ';'

endif
